@@ -1,11 +1,16 @@
 """Utility functions for creating an interpreter."""
 
+from collections import OrderedDict
+from copy import deepcopy
 from io import StringIO
 import os
 import re
+from functools import partial
 from tempfile import TemporaryDirectory
 from typing import List
 
+from IPython.core.display import display
+
 from robot.api import get_model
 from robot.errors import DataError
 from robot.reporting import ResultWriter
@@ -15,6 +20,8 @@ from robot.running.builder.parsers import ErrorReporter
 from robot.running.builder.transformers import SettingsBuilder, SuiteBuilder
 from robot.model.itemlist import ItemList
 
+from ipywidgets import VBox, HBox, Button, Output, Text
+
 from .utils import (
     detect_robot_context, line_at_cursor, scored_results,
     complete_libraries, get_lunr_completions, remove_prefix,
@@ -29,10 +36,146 @@ from .constants import VARIABLE_REGEXP, BUILTIN_VARIABLES
 from .listeners import RobotKeywordsIndexerListener
 
 
+def normalize_argument(name):
+    if "=" in name:
+        name, default = name.split("=", 1)
+    else:
+        default = None
+
+    return (
+        name,
+        re.sub(r"\W", "_", re.sub(r"^[^\w]*|[^\w]*$", "", name, re.U), re.U),
+        default
+    )
+
+
+def execute_keyword(suite: TestSuite, name, arguments, **values):
+    header = suite.rpa and "Tasks" or "Test Cases"
+    code = f"""\
+
+*** {header} ***
+
+{name}
+    {name}  {'  '.join([values[a[1]] for a in arguments])}
+"""
+
+    # Copy the test suite
+    suite = deepcopy(suite)
+    suite.rpa = True
+
+    with TemporaryDirectory() as path:
+        _, report = _execute_impl(code, suite, outputdir=path, interactive_keywords=False)
+
+    if report is not None:
+        display(report, raw=True)
+
+
+def on_button_execute(execute, controls, out, widgets, *args, **kwargs):
+    values = {key: control.value for key, control in controls.items()}
+
+    with out:
+        description = widgets[0].description
+        widgets[0].description = "Executing..."
+
+        for widget in widgets:
+            widget.disabled = True
+
+        out.clear_output(wait=True)
+
+        try:
+            execute(**values)
+        finally:
+            widgets[0].description = description
+            for widget in widgets:
+                widget.disabled = False
+
+
+def get_interactive_keyword(suite: TestSuite, keyword):
+    """Get an interactive widget for testing a keyword."""
+    name = keyword.name
+    arguments = [normalize_argument(arg) for arg in keyword.args]
+
+    # Make a copy of the suite, the suite the widget operates on must no be
+    # the same as the main one
+    suite_copy = deepcopy(suite)
+
+    execute_key = partial(execute_keyword, suite_copy, name, arguments)
+
+    widgets = []
+    controls = OrderedDict()
+    out = Output()
+
+    for arg in arguments:
+        input_widget = Text(description=arg[1] + "=", value=arg[2])
+        widgets.append(input_widget)
+        controls[arg[1]] = input_widget
+
+    button = Button(description=name)
+    button.on_click(partial(on_button_execute, execute_key, controls, out, widgets))
+    widgets.insert(0, button)
+
+    return VBox((HBox(widgets), out))
+
+
 class TestSuiteError(Exception):
     pass
 
 
+class ProgressUpdater(StringIO):
+    """Wrapper designed to capture robot.api.logger.console and display it.
+    This can be used passing an instance of this to the execute's stdout argument"""
+
+    colors = re.compile(r"\[[0-?]+[^m]+m")
+
+    def __init__(self, display, update_display):
+        self.display = display
+        self.update_display = update_display
+
+        self.progress = {"test": "n/a", "keyword": "n/a", "message": None}
+        self.already_displayed = False
+
+        super(ProgressUpdater, self).__init__()
+
+    def _update(self):
+        status_line = " | ".join(
+            str(s)
+            for s in [
+                self.progress["test"],
+                self.progress["keyword"],
+                self.progress["message"],
+            ]
+            if s
+        )
+
+        mimebundle = {
+            "text/html": f'<pre style="white-space:nowrap;overflow:hidden;padding-left:1ex;'
+                         f'"><i class="fa fa-spinner fa-pulse"></i>{status_line}</pre>'
+        }
+
+        if not self.already_displayed:
+            self.display(mimebundle)
+            self.already_displayed = True
+        else:
+            self.update_display(mimebundle)
+
+    def update(self, data):
+        if "test" in data:
+            self.progress["test"] = data["test"]
+            self.progress["message"] = None
+        elif "keyword" in data:
+            self.progress["keyword"] = data["keyword"]
+            self.progress["message"] = None
+        self._update()
+
+    def clear(self):
+        self.update_display({"text/plain": ""})
+
+    def write(self, s):
+        self.progress["message"] = s.strip()
+        self._update()
+        return super(ProgressUpdater, self).write(s)
+
+
 class ErrorStream():
     def __init__(self):
         self.message = ''
@@ -94,7 +237,7 @@ def generate_report(suite: TestSuite, outputdir: str):
 
 
 def _execute_impl(code: str, suite: TestSuite, defaults: TestDefaults = TestDefaults(),
-                  stdout=None, stderr=None, listeners=[], drivers=[], outputdir=None):
+                  stdout=None, stderr=None, listeners=[], drivers=[], outputdir=None, interactive_keywords=True):
     # Clear selector completion highlights
     for driver in yield_current_connection(drivers, ["RPA.Browser", "selenium", "jupyter"]):
         try:
@@ -121,6 +264,11 @@ def _execute_impl(code: str, suite: TestSuite, defaults: TestDefaults = TestDefa
     strip_duplicate_items(suite.resource.variables)
     strip_duplicate_items(suite.resource.keywords)
 
+    # If there is no test, allow the user to interact with defined keywords by providing widgets
+    new_keywords = [item for item in get_items_copy(suite.resource.keywords) if item not in keywords]
+    if not suite.tests and new_keywords and interactive_keywords:
+        return None, [get_interactive_keyword(suite, keyword) for keyword in new_keywords]
+
     # Set default streams
     # By default stdout is no-op
     # By default stderr raises an exception when flushing (workaround robotframework which does not raise)
@@ -162,7 +310,10 @@ def _execute_impl(code: str, suite: TestSuite, defaults: TestDefaults = TestDefa
 
 def execute(code: str, suite: TestSuite, defaults: TestDefaults = TestDefaults(),
             stdout=None, stderr=None, listeners=[], drivers=[], outputdir=None):
-    """Execute a snippet of code, given the current test suite."""
+    """
+    Execute a snippet of code, given the current test suite. Returns a tuple containing the result of the
+    suite (if there were tests) and a displayable object containing either the report or interactive widgets.
+    """
     if outputdir is None:
         with TemporaryDirectory() as path:
             result = _execute_impl(code, suite, defaults, stdout, stderr, listeners, drivers, path)
